import pytest

from zk_chat.markdown.loader import split_metadata_and_content


@pytest.fixture
def file_content_with_json_metadata():
    return """---
{
    "title": "Sample Document",
    "author": "John Doe"
}
---
This is the content of the document."""


@pytest.fixture
def file_content_with_yaml_metadata():
    return """---
title: Sample Document
author: John Doe
---
This is the content of the document."""


@pytest.fixture
def file_content_with_incorrect_metadata_start_marker():
    return """--
title: Sample Document
author: John Doe
---
This is the content of the document."""


@pytest.fixture
def file_content_with_incorrect_metadata_end_marker():
    return """---
title: Sample Document
author: John Doe
--
This is the content of the document."""


@pytest.fixture
def file_content_without_metadata():
    return "This is the content of the document without metadata."


@pytest.fixture
def file_content_with_no_content():
    return """---
{
    "title": "Sample Document",
    "author": "John Doe"
}
---"""


@pytest.fixture
def file_content_with_no_metadata_and_separators_in_body_content():
    return """This is part one of the content.
---
This is part two of the content."""


@pytest.fixture
def file_content_empty():
    return ""


def test_split_metadata_and_content_with_json_metadata(file_content_with_json_metadata):
    expected_metadata = {
        "title": "Sample Document",
        "author": "John Doe"
    }
    expected_content = "This is the content of the document."
    metadata, content = split_metadata_and_content(file_content_with_json_metadata)
    assert metadata == expected_metadata
    assert content == expected_content


def test_split_metadata_and_content_with_yaml_metadata(file_content_with_yaml_metadata):
    expected_metadata = {
        "title": "Sample Document",
        "author": "John Doe"
    }
    expected_content = "This is the content of the document."
    metadata, content = split_metadata_and_content(file_content_with_yaml_metadata)
    assert metadata == expected_metadata
    assert content == expected_content


def test_split_metadata_and_content_with_incorrect_metadata_start_marker(
        file_content_with_incorrect_metadata_start_marker):
    expected_metadata = {}
    expected_content = """--
title: Sample Document
author: John Doe
---
This is the content of the document."""
    metadata, content = split_metadata_and_content(file_content_with_incorrect_metadata_start_marker)
    assert metadata == expected_metadata
    assert content == expected_content


def test_split_metadata_and_content_with_incorrect_metadata_end_marker(
        file_content_with_incorrect_metadata_end_marker):
    expected_metadata = {}
    expected_content = """---
title: Sample Document
author: John Doe
--
This is the content of the document."""
    metadata, content = split_metadata_and_content(file_content_with_incorrect_metadata_end_marker)
    assert metadata == expected_metadata
    assert content == expected_content


def test_split_metadata_and_content_without_metadata(file_content_without_metadata):
    expected_metadata = {}
    expected_content = "This is the content of the document without metadata."
    metadata, content = split_metadata_and_content(file_content_without_metadata)
    assert metadata == expected_metadata
    assert content == expected_content


def test_split_metadata_and_content_with_no_content(file_content_with_no_content):
    expected_metadata = {
        "title": "Sample Document",
        "author": "John Doe"
    }
    expected_content = ""
    metadata, content = split_metadata_and_content(file_content_with_no_content)
    assert metadata == expected_metadata
    assert content == expected_content


def test_split_metadata_and_content_with_empty_file(file_content_empty):
    expected_metadata = {}
    expected_content = ""
    metadata, content = split_metadata_and_content(file_content_empty)
    assert metadata == expected_metadata
    assert content == expected_content


def test_split_metadata_and_content_with_no_metadata_and_separators_in_body_content(
        file_content_with_no_metadata_and_separators_in_body_content):
    expected_metadata = {}
    expected_content = """This is part one of the content.
---
This is part two of the content."""
    metadata, content = split_metadata_and_content(file_content_with_no_metadata_and_separators_in_body_content)
    assert metadata == expected_metadata
    assert content == expected_content



import os
import re

from pydantic import BaseModel


class ZkDocument(BaseModel):
    relative_path: str
    metadata: dict
    content: str

    @property
    def title(self) -> str:
        return self.strip_identifier_prefix(self.base_filename_without_extension())

    @property
    def id(self) -> str:
        return self.relative_path

    def strip_identifier_prefix(self, string):
        return re.sub(r'^[@!]\s*', '', string)

    def base_filename_without_extension(self):
        return os.path.splitext(os.path.basename(self.relative_path))[0]


class ZkDocumentChunk(BaseModel):
    document_id: str
    document_title: str
    text: str


class ZkQueryResult(BaseModel):
    chunk: ZkDocumentChunk
    distance: float



import configparser
import os

CONFIG_PATH = os.path.expanduser("~/.zk_chat")


def load_or_initialize_config():
    config = configparser.ConfigParser()
    if os.path.exists(CONFIG_PATH):
        config.read(CONFIG_PATH)
        vault = config.get('zk_chat', 'vault')
        model = config.get('zk_chat', 'model')
    else:
        vault = input("Enter path to your zettelkasten vault: ")
        model = input("Enter LLM model to use: ")
        config['zk_chat'] = {'vault': vault, 'model': model}
        with open(CONFIG_PATH, 'w') as f:
            config.write(f)
    return vault, model



from zk_chat.rag.splitter import split_to_chunks


def test_split_to_chunks_exact_chunk_size():
    tokens = list(range(500))
    chunk_size = 500
    chunk_overlap = 100
    expected_chunks = [tokens]
    chunks = split_to_chunks(tokens, chunk_size, chunk_overlap)
    assert chunks == expected_chunks


def test_split_to_chunks_with_overlap():
    tokens = list(range(600))
    chunk_size = 500
    chunk_overlap = 100
    expected_chunks = [tokens[:500], tokens[400:600]]
    chunks = split_to_chunks(tokens, chunk_size, chunk_overlap)
    assert chunks == expected_chunks


def test_split_to_chunks_no_overlap():
    tokens = list(range(1000))
    chunk_size = 500
    chunk_overlap = 0
    expected_chunks = [tokens[:500], tokens[500:1000]]
    chunks = split_to_chunks(tokens, chunk_size, chunk_overlap)
    assert chunks == expected_chunks


def test_split_to_chunks_smaller_than_chunk_size():
    tokens = list(range(300))
    chunk_size = 500
    chunk_overlap = 100
    expected_chunks = [tokens]
    chunks = split_to_chunks(tokens, chunk_size, chunk_overlap)
    assert chunks == expected_chunks


def test_split_to_chunks_with_remainder():
    tokens = list(range(750))
    chunk_size = 500
    chunk_overlap = 100
    expected_chunks = [tokens[:500], tokens[400:750]]
    chunks = split_to_chunks(tokens, chunk_size, chunk_overlap)
    assert chunks == expected_chunks


def test_split_to_chunks_empty_tokens():
    tokens = []
    chunk_size = 500
    chunk_overlap = 100
    expected_chunks = []
    chunks = split_to_chunks(tokens, chunk_size, chunk_overlap)
    assert chunks == expected_chunks



import hashlib
import os
from typing import List

from mojentic.llm.gateways.embeddings_gateway import EmbeddingsGateway
from mojentic.llm.gateways.tokenizer_gateway import TokenizerGateway

from zk_chat.chroma_gateway import ChromaGateway
from zk_chat.markdown.loader import load_markdown
from zk_chat.models import ZkDocument, ZkDocumentChunk, ZkQueryResult
from zk_chat.rag.splitter import split_to_chunks


class Zettelkasten:
    def __init__(self, root_path: str, embeddings_gateway: EmbeddingsGateway, tokenizer_gateway: TokenizerGateway,
                 document_db: ChromaGateway):
        self.root_path = root_path
        self.embeddings_gateway: EmbeddingsGateway = embeddings_gateway
        self.tokenizer_gateway: TokenizerGateway = tokenizer_gateway
        self.document_db: ChromaGateway = document_db
        self.document_cache = None

    def all_markdown_documents(self) -> List[ZkDocument]:
        if not self.document_cache:
            documents = []
            for dirpath, _, filenames in os.walk(self.root_path):
                for filename in filenames:
                    if filename.endswith(".md"):
                        full_path = os.path.join(dirpath, filename)
                        document = self.read_zk_document(full_path)
                        documents.append(document)
            self.document_cache = documents
        return self.document_cache

    def read_zk_document(self, full_path: str) -> ZkDocument:
        relative_path = os.path.relpath(full_path, self.root_path)
        (metadata, content) = load_markdown(full_path)
        document = ZkDocument(
            relative_path=relative_path,
            metadata=metadata,
            content=content
        )
        return document

    # How long will it be feasible to just load this all into RAM?
    def find_by_id(self, id: str) -> ZkDocument:
        return next((document for document in self.all_markdown_documents() if document.id == id), None)

    def chunk_and_index(self, chunk_size=200, chunk_overlap=20):
        self.document_db.reset_indexes()
        documents = self.all_markdown_documents()
        for document in documents:
            self._chunk_document(document, chunk_size, chunk_overlap)

    def query_chunks(self, query: str, n_results: int = 5, max_distance: float = 1.0) -> List[ZkQueryResult]:
        query_embedding = self.embeddings_gateway.calculate(query)
        response = self.document_db.query(query_embedding, n_results=n_results)
        # print(response.keys()) # ids, embeddings, documents, uris, data, metadata, distances, included
        results = []
        for i, distance in enumerate(response['distances'][0]):
            if distance <= max_distance:
                chunk = ZkDocumentChunk(
                    document_id=response['metadatas'][0][i]['id'],
                    document_title=response['metadatas'][0][i]['title'],
                    text=response['documents'][0][i]
                )
                results.append(ZkQueryResult(chunk=chunk, distance=distance))
        return results

    def _chunk_document(self, document, chunk_size=200, chunk_overlap=20):
        print(f"Processing {document.title}")
        tokens = self.tokenizer_gateway.encode(document.content)
        print(f"    Content is {len(document.content)} bytes, or {len(tokens)} tokens long")
        token_chunks = split_to_chunks(tokens, chunk_size=chunk_size, chunk_overlap=chunk_overlap)
        if len(token_chunks) > 0:
            print(f"    Split into {len(token_chunks)} chunks")
            print([len(chunk) for chunk in token_chunks])

            text_chunks = self._decode_tokens_to_text(token_chunks)

            self._add_text_chunks_to_index(document, text_chunks)

    def _add_text_chunks_to_index(self, document, text_chunks):
        self.document_db.add_items(
            ids=[hashlib.md5(bytes(hunk, "utf-8")).hexdigest() for hunk in text_chunks],
            documents=text_chunks,
            metadatas=[{
                "id": document.id,
                "title": document.title,
            } for _ in text_chunks],
            embeddings=[self.embeddings_gateway.calculate(chunk) for chunk in text_chunks]
        )

    def _decode_tokens_to_text(self, token_chunks):
        return [self.tokenizer_gateway.decode(chunk) for chunk in token_chunks]



from mojentic.llm.gateways.embeddings_gateway import EmbeddingsGateway
from mojentic.llm.gateways.tokenizer_gateway import TokenizerGateway

from zk_chat.chroma_gateway import ChromaGateway
from zk_chat.config import load_or_initialize_config
from zk_chat.zettelkasten import Zettelkasten


def reindex(vault):
    chroma = ChromaGateway()
    zk = Zettelkasten(root_path=vault, embeddings_gateway=EmbeddingsGateway(), tokenizer_gateway=TokenizerGateway(),
                      document_db=chroma)

    zk.chunk_and_index(chunk_size=200, chunk_overlap=20)


def main():
    print("Running zk_reindex main function")
    vault, model = load_or_initialize_config()
    reindex(vault)


if __name__ == '__main__':
    main()



import logging

from mojentic.llm import LLMBroker, ChatSession
from mojentic.llm.gateways.embeddings_gateway import EmbeddingsGateway
from mojentic.llm.gateways.tokenizer_gateway import TokenizerGateway

from config import load_or_initialize_config
from zk_chat.chroma_gateway import ChromaGateway
from zk_chat.rag import rag_query
from zk_chat.zettelkasten import Zettelkasten

logging.basicConfig(
    level=logging.WARN
)


def chat(vault, model):
    chroma = ChromaGateway()
    zk = Zettelkasten(root_path=vault, embeddings_gateway=EmbeddingsGateway(), tokenizer_gateway=TokenizerGateway(),
                      document_db=chroma)
    llm = LLMBroker(model)

    chat_session = ChatSession(llm, system_prompt="You are a helpful research assistant.")

    while True:
        query = input("Query: ")
        if not query:
            break
        else:
            response = rag_query(chat_session, zk, query)
            print(response)


def main():
    print("Running zk_chat main function")
    vault, model = load_or_initialize_config()
    chat(vault, model)


if __name__ == '__main__':
    main()



import pytest

from zk_chat.markdown.loader import parse_metadata


@pytest.fixture
def json_metadata_str():
    return '{"title": "Sample Document", "author": "John Doe"}'


@pytest.fixture
def yaml_metadata_str():
    return 'title: Sample Document\nauthor: John Doe'


@pytest.fixture
def invalid_metadata_str():
    return 'title: Sample Document\nauthor: John Doe\n---\nInvalid'


@pytest.fixture
def empty_metadata_str():
    return ''


def test_parse_metadata_with_json(json_metadata_str):
    expected_metadata = {
        "title": "Sample Document",
        "author": "John Doe"
    }
    metadata = parse_metadata(json_metadata_str)
    assert metadata == expected_metadata


def test_parse_metadata_with_yaml(yaml_metadata_str):
    expected_metadata = {
        "title": "Sample Document",
        "author": "John Doe"
    }
    metadata = parse_metadata(yaml_metadata_str)
    assert metadata == expected_metadata


def test_parse_metadata_with_invalid_metadata(invalid_metadata_str):
    expected_metadata = {}
    metadata = parse_metadata(invalid_metadata_str)
    assert metadata == expected_metadata


def test_parse_metadata_with_empty_metadata(empty_metadata_str):
    expected_metadata = {}
    metadata = parse_metadata(empty_metadata_str)
    assert metadata == expected_metadata



from setuptools import setup, find_packages

# Read dependencies from requirements.txt
with open("requirements.txt", "r") as req_file:
    requirements = [line.strip() for line in req_file if line.strip()
                    and not line.startswith("#")]

setup(
    name='zk-rag',
    version='1.0.1',
    packages=find_packages(),  # replaced py_modules with packages=find_packages()
    install_requires=requirements,  # added to install modules from requirements.txt
    entry_points={
        'console_scripts': [
            'zk_reindex=zk_chat.reindex:main',  # updated entry point for root modules
            'zk_chat=zk_chat.chat:main'         # updated entry point for root modules
        ]
    },
)
